package com.halomobi.common;

import android.content.Context;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.content.res.Configuration;
import android.text.TextUtils;

import com.halomobi.utils.SDKInfo;
import com.halomobi.utils.SystemUtils;
import com.halomobi.utils.Utils;

import java.util.UUID;


public class Config {

    /**
     * 广告请求超时时间
     */
    public static final long AD_REQUEST_TIME_OUT = 3 * 1000;
    /**
     * 广告渲染超时时间
     */
    public static final long AD_RENDER_TIME_OUT = 2 * 1000;
    /**
     * 检测View遮挡时间
     */
    public static final long CHECK_VIEW_TIME = 3 * 1000;
    /**
     * Banner 轮播间隔
     */
    public static long BANNER_REQUEST_INTERVAL = 30 * 1000;
    /**
     * 悬浮按钮 轮播间隔
     */
    public static long FLOAT_REQUEST_INTERVAL = 30 * 1000;


    /**
     * 操作系统
     */
    public static final int OS_TYPE = 2;
    /**
     * SDK宿主APP版本
     */
    private static String mAppVersion;
    /**
     * SDK宿主APP包名
     */
    private static String mAppPackageName = "";
    /**
     * SDK宿主APP名字
     */
    private static String mAppName;
    /**
     * 设备类型
     */
    private static int mDeviceType;
    /**
     * 操作系统 版本
     */
    private static String mOsVersion;
    /**
     * 手机品牌
     */
    private static String mVendor;
    /**
     * 手机型号
     */
    private static String mDeviceModel;
    /**
     * 安卓ID
     */
    private static String mAndroidID;
    /**
     * 手机IMEI串号
     */
    private static String mImei;
    /**
     * 手机IMEI串号的MD5
     */
    private static String mImeiMD5;
    /**
     * 手机IMSI
     */
    private static String mImsi;
    /**
     * mac地址
     */
    private static String mMacAddress;
    /**
     * 系统语言
     */
    private static String mLanguage;
    /**
     * 手机分辨率
     */
    private static String mResolution;
    // Device

    //以下为新加参数
    /**
     * 是否root
     */
    private static int mIsRoot = -1;
    /**
     * 蓝牙mac
     */
    private static String mBluetoothMac;
    /**
     * pseudo-unique ID
     */
    private static String mSerial;
    /**
     * cpu类型
     */
    private static String mCpuModel;
    /**
     * CPU序列号
     */
    private static String mCpuSerial;
    /**
     * SDK 名字
     */
    public static String SDK_NAME = SDKInfo.SDK;
    /**
     * SDK渠道信息
     */
    public static String SDK_CHANNEL = SDKInfo.CHANNEL_ID;

    //公参
    /**
     * 设备标识
     */
    private static String mDevice;
    /**
     * 用户标识
     */
    private static String mCuid;
    /**
     * 终端ID
     */
    private static String mClientID;
    /**
     * 设备ID
     */
    private static String mDeviceID;
    /**
     * Android系统版本对应码
     */
    private static String mOsVersionInt;
    private static float mDpi = -1;
    private static double mDeviceDpi = -1;


    //公参
    /**
     * SDK宿主APPID
     */
    public static String mAppId = "";
    /**
     * SDK版本
     */
    public static String SDK_VERSION = SDKInfo.SDK_VERSION;
    //当前网络环境（测试、线上等）
    private static NetEnvironment sNetEnvironment = NetEnvironment.ONLINE;

    /**
     * SDK宿主APP版本
     */
    public static String getAppVersion() {
        if (TextUtils.isEmpty(mAppVersion)) {
            mAppVersion = SystemUtils.getAppVer();
        }
        return mAppVersion;
    }

    /**
     * SDK宿主APP包名
     */
    public static String getAppPackageName() {
        if (TextUtils.isEmpty(mAppPackageName) && Utils.getContext() != null) {
            mAppPackageName = Utils.getContext().getPackageName();
        }
        return mAppPackageName;
    }

    public static String getAppName(String packageName) {
        if (TextUtils.isEmpty(mAppName)) {
            PackageManager pm = Utils.getContext().getPackageManager();
            try {
                PackageInfo info = pm.getPackageInfo(packageName,PackageManager.GET_ACTIVITIES);
                mAppName = info.applicationInfo.loadLabel(pm).toString();
            } catch (PackageManager.NameNotFoundException e) {
                e.printStackTrace();
            }
        }
        return mAppName;
    }

    /**
     * 设备类型
     */
    public static int getDeviceType() {
        if (mDeviceType == 0) {
            mDeviceType = SystemUtils.getDeviceType();
        }
        return mDeviceType;
    }

    /**
     * 操作系统 版本
     */
    public static String getOSVersion() {
        if (TextUtils.isEmpty(mOsVersion)) {
            mOsVersion = SystemUtils.getOSVersion();
        }
        return mOsVersion;
    }

    /**
     * 手机品牌
     */
    public static String getVendor() {
        if (TextUtils.isEmpty(mVendor)) {
            mVendor = SystemUtils.getVendor();
        }
        return mVendor;
    }

    /**
     * 手机型号
     */
    public static String getDeviceModel() {
        if (TextUtils.isEmpty(mDeviceModel)) {
            mDeviceModel = SystemUtils.getDeviceModel();
        }
        return mDeviceModel;
    }

    /**
     * 安卓ID
     */
    public static String getAndroidID() {
        mAndroidID = SystemUtils.getInfoByJson("dpid");
//        if (TextUtils.isEmpty(mAndroidID)) {
//            mAndroidID = SystemUtils.getAndroidID();
//        }
        return mAndroidID;
    }

    /**
     * 手机imei串号
     */
    public static String getIMEI() {
        mImei = SystemUtils.getInfoByJson("did");
//        if (TextUtils.isEmpty(mImei)) {
//            mImei = SystemUtils.getIMEI(3);
//        }
        return mImei;
    }

    /**
     * mac地址
     */
    public static String getMacAddress() {
        String mac = SystemUtils.getInfoByJson("mac");
        return mac;
//        mMacAddress = SystemUtils.getInfoByJson("mac");
//        if (TextUtils.isEmpty(mMacAddress)) {
//            mMacAddress = NetworkUtils.getMacAddress();
//        }
//        return mMacAddress;
    }

    /**
     * 设备标识
     */
    public static String getDevice() {
        if (TextUtils.isEmpty(mDevice)) {
            mDevice = SystemUtils.getDevice();
        }
        return mDevice;
    }

    /**
     * Android系统版本对应码
     */
    public static String getOsVersionInt() {
        if (TextUtils.isEmpty(mOsVersionInt)) {
            mOsVersionInt = SystemUtils.getSDK_INT() + "";
        }
        return mOsVersionInt;
    }

    public static float getDpi() {
        if (mDpi == -1) {
            mDpi = SystemUtils.getDPI();
        }
        return mDpi;
    }

    public static double getDeviceDpi() {
        if (mDeviceDpi == -1) {
            mDeviceDpi = SystemUtils.getDeviceDPI();
        }
        return mDeviceDpi;
    }

    /**
     * 系统语言
     */
    public static String getLanguage() {
        if (TextUtils.isEmpty(mLanguage)) {
            mLanguage = SystemUtils.getLanguage();
        }
        return mLanguage;
    }

    /**
     * 手机分辨率
     */
    public static String getResolution() {
        if (TextUtils.isEmpty(mResolution)) {
            mResolution = SystemUtils.getResolution();
        }
        return mResolution;
    }

    /**
     * 是否root
     */
    public static int isRoot() {
        if (mIsRoot == -1) {
            mIsRoot = SystemUtils.isRoot() ? 1 : 0;
        }
        return mIsRoot;
    }

    /**
     * 蓝牙mac
     */
    public static String getBluetoothMac() {
        if (TextUtils.isEmpty(mBluetoothMac)) {
            mBluetoothMac = SystemUtils.getBluetoothMac();
        }
        return mBluetoothMac;
    }

    /**
     * pseudo-unique ID
     */
    public static String getSerial() {
        if (TextUtils.isEmpty(mSerial)) {
            mSerial = SystemUtils.getSerial();
        }
        return mSerial;
    }

    /**
     * 获取CPU类型
     */
    public static String getCpuModel() {
        if (TextUtils.isEmpty(mCpuModel)) {
            mCpuModel = SystemUtils.getCpuModel();
        }
        return mCpuModel;
    }

    /**
     * CPU序列号
     */
    public static String getCPUSerial() {
        if (TextUtils.isEmpty(mCpuSerial)) {
            mCpuSerial = SystemUtils.getCPUSerial();
        }
        return mCpuSerial;
    }

    /**
     * 随机唯一id
     */
    public static String getUUID() {
        return UUID.randomUUID().toString().replace("-", "");
    }

    /**
     * 初始化
     * .
     **/
    public static void init(String appID) {
        mAppId = appID;
        SDK_VERSION = SDKInfo.SDK_VERSION;
        SystemUtils.initUserAgent();
    }

    /**
     * 设置SDK版本号
     *
     * @param version 最新版本号
     */
    public static void setSdkVersion(String version) {
        SDK_VERSION = version;
    }

    /**
     * 获取网络当前网络环境（测试、线上等）
     *
     * @return 当前网络环境
     */
    public static NetEnvironment getNetEnvironment() {
        return sNetEnvironment;
    }

    public static void setNetEnvironment(NetEnvironment netEnvironment){
        sNetEnvironment=netEnvironment;
    }

    public static boolean isTabletDevice(Context context) {
        return (context.getResources().getConfiguration().screenLayout & Configuration.SCREENLAYOUT_SIZE_MASK) >=
                Configuration.SCREENLAYOUT_SIZE_LARGE;
    }
}
