package com.halomobi.network.net.http;

import android.text.TextUtils;

import com.halomobi.utils.FileUtils;
import com.halomobi.utils.Utils;

import java.io.IOException;
import java.io.InputStream;
import java.net.HttpURLConnection;

/**
 * Created by liuzheng on 2017/2/28.
 */

public class HttpResult {

    private static final String[] httpCode3XXs = {"多选项", "永久移动", "临时移动",
            "参见其他", "未改动", "使用代理", "", "暂时重定向"};
    private static final String[] httpCode4XXs = {"错误请求", "未授权", "要求付费", "禁止",
            "未找到", "不允许的方法", "不被采纳", "要求代理授权", "请求超时", "冲突", "过期的", "要求的长度",
            "前提不成立", "请求实例太大", "请求URI太大", "不支持的媒体类型", "无法满足的请求范围", "失败的预期"};
    private static final String[] httpCode5XXs = {"内部服务器错误", "未被使用", "网关错误",
            "不可用的服务", "网关超时", "HTTP版本未被支持"};

    private HttpURLConnection mConnection;
    private InputStream mIn;
    private String mStr;
    private int mCode;

    public HttpResult(HttpURLConnection connection) throws IOException {
        mConnection = connection;
        mCode = connection.getResponseCode();
    }

    /**
     * @param statusCode 状态码
     * @return ErrorMsg
     * @since 2015-12-9
     */
    public static String getMsgByErrorCode(int statusCode) {
        String resultMsg = "";
        if (300 <= statusCode && statusCode < 400) {
            resultMsg = "重定向:" + statusCode;
            int code = statusCode - 300;
            if (code < httpCode3XXs.length) {
                resultMsg += "----->" + httpCode3XXs[code];
            }
        }
        if (400 <= statusCode && statusCode < 500) {
            resultMsg = "客户端错误:" + statusCode;
            int code = statusCode - 400;
            if (code < httpCode4XXs.length) {
                resultMsg += "----->" + httpCode4XXs[code];
            }
        } else if (500 <= statusCode) {
            resultMsg = "服务器错误:" + statusCode;
            int code = statusCode - 500;
            if (code < httpCode5XXs.length) {
                resultMsg += "----->" + httpCode5XXs[code];
            }
        }
        return resultMsg;
    }

    public HttpURLConnection getHttpResponse() {
        return mConnection;
    }


    public int getCode() {
        return mCode;
    }

    /**
     * 从结果中获取字符串，一旦获取，会自动关流，并且把字符串保存，方便下次获取
     */
    public String getString() {
        if (!TextUtils.isEmpty(mStr)) {
            return mStr;
        }
        InputStream inputStream = getInputStream();

        String contentEncoding = mConnection.getContentEncoding();
        try {
            mStr = Utils.parseInputString(Utils.decodeInputStream(inputStream, contentEncoding));
//            Log.e("fffff", "===mStr===" + mStr);
            if (TextUtils.isEmpty(mStr)) {
                mStr = mConnection.getResponseMessage();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return mStr;
    }

    /**
     * 获取文件长度
     */
    public long getLength() {
        if (getCode() >= 200 && getCode() < 400) {
            return mConnection.getContentLength();
        }
        return 0;
    }

    /**
     * 获取流，需要使用完毕后调用close方法关闭网络连接
     */
    public InputStream getInputStream() {
        if (mIn == null && getCode() < 400) {
            try {
                mIn = mConnection.getInputStream();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return mIn;
    }

    /**
     * 关闭网络连接
     */
    public void close() {
        FileUtils.close(mIn);
        if (mConnection != null) {
            mConnection.disconnect();
        }
    }
}
