package com.halomobi.network.report.track;


import static com.halomobi.common.ConstantPool.EroType.BAD_TRACK;

import com.halomobi.network.listeners.TrackerTaskListener;
import com.halomobi.network.net.AbsNetProcessor;
import com.halomobi.network.net.AbsNetTask;
import com.halomobi.network.net.http.HttpResult;
import com.halomobi.utils.LogUtils;

import java.util.Set;

class ReportTrackerProcessor extends AbsNetProcessor {

    private TrackerTaskListener mListener;

    private boolean isSuccess = true;

    private ReportTrackerTask mTask;

    /**
     * 是否是上报之前失败的监播历史数据
     */
    private boolean isReportHistory = false;

    ReportTrackerProcessor(Set<String> trackUrls, TrackerTaskListener listener, boolean isHistory) {
        mListener = listener;
        isReportHistory = isHistory;

        mTask = new ReportTrackerTask(trackUrls);
    }

    /**
     * 创建TASK
     *
     * @return 错误上报NetTask
     */
    @Override
    protected AbsNetTask createNetTask() {
        return mTask;
    }

    /**
     * 网络请求成功返回
     */
    @Override
    public void onDataReceived(HttpResult httpResult) throws Exception {
        if (httpResult.getCode() < 300) {
            isSuccess &= true;
        } else {
            LogUtils.e("  httpCode：" + httpResult.getCode());
            reportError(BAD_TRACK, "监播发送失败 , " + "  httpCode：" + httpResult.getCode() + " , URL：" + mTask.getUrl());
            isSuccess = false;
        }

        if (!mTask.isLastRequest()) {
            //如果不是一轮监播中的最后一条，直接返回
            return;
        }

        if (isSuccess) {
            //所有监播都发送完成且没有失败才会触发banner轮播，如果mListener != null 说明是展示监播。
            onTrackersReturn();
        }

        //每一轮监播发送成功后检测是否有失败监播可以发送
//        if (!isReportHistory) {//避免陷入无限循环，只触发一次上报历史
//            ReportTrackerManager.getInstance().reportTrackerCache();//检测是否有失败监播可以发送
//        }

    }

    /**
     * 网络请求错误返回
     */
    @Override
    public void onErrorReceived(String eroMsg, int eroCode) {
        reportError(BAD_TRACK, "监播发送失败, error_msg : " + eroMsg + "  \nURL：" + mTask.getUrl());
        //Listener是为了回调到Banner类触发轮播的，一系列监播中任何一次发生错误，就停止触发Banner轮播，
        isSuccess = false;

        //每一轮监播发送成功后检测是否有失败监播可以发送
//        if (!isReportHistory && mTask.isLastRequest()) {//避免陷入无限循环，只触发一次上报历史
//            ReportTrackerManager.getInstance().reportTrackerCache();//检测是否有失败监播可以发送
//        }
    }

    /**
     * 网络错误
     */
    @Override
    public void onNetworkError() {
        reportError(BAD_TRACK, "监播发送失败,网络错误，URL：" + mTask.getUrl());
        //Listener是为了回调到Banner类触发轮播的，一系列监播中任何一次发生错误，就停止触发Banner轮播
        isSuccess = false;
    }

    /**
     * 回调到Manager
     */
    private void onTrackersReturn() {
        if (mListener == null) {
            return;
        }

        mListener.onTrackersReturn();
    }

    /**
     * 监播发送失败，上报错误信息
     *
     * @param error_code 错误码
     * @param error_msg  错误信息
     */
    private void reportError(int error_code, String error_msg) {
//        ReportTrackerManager.getInstance().addTracker(mTask.getUrl());
//        ReportErrorManager.getInstance().reportError(new ErrorInfo(), error_code, error_msg);
    }

}
