package com.halomobi.network.request.ad;

import android.media.MediaMetadataRetriever;
import android.text.TextUtils;
import android.util.Log;
import com.halomobi.common.ConstantPool;
import com.halomobi.network.listeners.AdStateChangListener;
import com.halomobi.network.net.AbsNetProcessor;
import com.halomobi.network.net.AbsNetTask;
import com.halomobi.network.net.http.HttpResult;
import com.halomobi.network.request.ad.bean.Ad;
import com.halomobi.network.request.ad.bean.AdParameter;
import com.halomobi.utils.LogUtils;
import com.halomobi.utils.NetworkUtils;
import com.halomobi.utils.Utils;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;

public class RequestAdProcessor extends AbsNetProcessor {
    private AdStateChangListener mListener;
    private AdParameter mAdParameter;

    RequestAdProcessor(AdParameter adParameter, AdStateChangListener listener) {
        mListener = listener;
        mAdParameter = adParameter;
    }

    @Override
    public void onNetworkError() {
        onAdLoadFailed("网络异常", ConstantPool.EroType.WITHOUT_NET);
    }

    @Override
    public void onErrorReceived(String eroMsg, int eroCode) {
        onAdLoadFailed(eroMsg, eroCode);
    }

    @Override
    public void onDataReceived(HttpResult result) throws Exception {
        ArrayList<Ad> adms = getBeanFromJson(result.getString());
        if (adms == null) {
            return;
        }

        if (adms.isEmpty()) {
            //没有返回广告数据集合
            onAdLoadFailed("广告返回空集合", ConstantPool.EroType.BACK_EMPTY);
            return;
        }

        Ad ad = adms.get(0);
        ad.response_time = System.currentTimeMillis();
        ad.setAdType(mAdParameter.adType.getType());

        if (mListener == null) {
            //成功返回广告
            LogUtils.d("回调Listener == null");
            return;
        }
        mListener.onAdReach(ad);
    }

    @Override
    protected AbsNetTask createNetTask() {
        return new RequestAdTask(mAdParameter);
    }

    private void onAdLoadFailed(final String errorMsg, final int errorCode) {
        Utils.post(new Runnable() {
            @Override
            public void run() {
                LogUtils.d("onAdLoadFailed errorCode:" + errorCode);
                onAdError(errorMsg, errorCode);
            }
        });
    }

    @Override
    public void process() {

        onStart();

        if (NetworkUtils.getNetworkType() == NetworkUtils.NET_NONE) {
            onAdLoadFailed("无网络", ConstantPool.EroType.WITHOUT_NET);
            return;
        }

        if (TextUtils.isEmpty(mAdParameter.adSlotId)) {
            onAdLoadFailed("广告 id 为空", ConstantPool.EroType.PARAMS_ERO);
            return;
        }

        super.process();

    }

    /**
     * 广告请求失败
     *
     * @param errorMsg  错误信息
     * @param errorCode 错误码
     */
    public void onAdError(String errorMsg, int errorCode) {
        if (mListener != null) {
            mListener.onAdError(errorMsg, errorCode);
        }
    }

    /**
     * 广告请求开始回调
     */
    private void onStart() {
        if (mListener != null) {
            mListener.onStart();
        }
    }

    /**
     * 解析json
     *
     * @param json json对象
     * @return 广告集合
     * @throws JSONException
     */
    private synchronized ArrayList<Ad> getBeanFromJson(String json) throws JSONException {
        LogUtils.e("==json===" + json);
        if (TextUtils.isEmpty(json)) {
            LogUtils.e("json 为空！");
            return null;
        }
        JSONObject jsonObject = new JSONObject(json);
        int error_code = jsonObject.optInt("errorcode");
        if (error_code != 0) {
            onAdLoadFailed("business_error_code:" + error_code, error_code);
            return null;
        }

        ArrayList<Ad> adms = new ArrayList<>();

        JSONArray jsonArray = jsonObject.optJSONArray("seatbid");
        if (jsonArray == null) {
            LogUtils.w("服务器返回的广告集合为null");
            return adms;
        }
        JSONObject bidObj = jsonArray.optJSONObject(0);
        JSONArray bidArr = bidObj.optJSONArray("bid");
        for (int i = 0; i < bidArr.length(); i++) {
            JSONObject object = bidArr.optJSONObject(i);
            if (object == null) {
                continue;
            }
            adms.add(parserAd(object));
        }

        return adms;
    }

    /**
     * 解析广告内容
     *
     * @throws JSONException
     */
    private Ad parserAd(JSONObject jsonAd) throws JSONException {
        Ad ad = new Ad();
        ad.id = jsonAd.optString("id");
        ad.impid = jsonAd.optString("impid");
        ad.price = jsonAd.optDouble("price");
        ad.adid = jsonAd.optString("adid");
        ad.nurl = jsonAd.optString("nurl");
        ad.burl = jsonAd.optString("burl");
        ad.adm = jsonAd.optString("adm");
        ad.w = jsonAd.optInt("w");
        ad.h = jsonAd.optInt("h");
        ad.crid = jsonAd.optString("crid");
        ad.landing_url = jsonAd.optString("lurl");
//        ad.imp_tracks = add2List(jsonAd, "imp_tracks");
//        ad.dp_tracks = add2List(jsonAd, "dp_tracks");
//        ad.click_tracks = add2List(jsonAd, "click_tracks");
//        ad.down_start_tracks = add2List(jsonAd, "down_start_tracks");
//        ad.down_complete_tracks = add2List(jsonAd, "down_complete_tracks");
//        ad.install_start_tracks = add2List(jsonAd, "install_start_tracks");
//        ad.install_complete_tracks = add2List(jsonAd, "install_complete_tracks");
//        ad.bundle = jsonAd.optString("bundle");
//        JSONObject logoObject = jsonAd.optJSONObject("logo");
//        if (logoObject != null) {
//            ad.lgsrc = logoObject.optString("url");
//        }
//        JSONObject imageObject = jsonAd.optJSONObject("image");
//        if (imageObject != null) {
//            ad.imgurl = imageObject.optString("url");
//            ad.w = imageObject.optInt("w");
//            ad.h = imageObject.optInt("h");
//        }
//        ad.title = jsonAd.optString("title");
//        ad.des = jsonAd.optString("des");
//        ad.is_download = jsonAd.optBoolean("is_download");
//        ad.clkurl = jsonAd.optString("file_url");
//        ad.landing_url = jsonAd.optString("landing_url");
//
//        ad.is_deep = jsonAd.optBoolean("is_deep");
//        ad.interaction_type = 1;
//        if (ad.is_deep) {
//            ad.dplk = jsonAd.optString("deeplink");
//        }
//        ad.macro = jsonAd.optInt("macro");
//        String source = jsonAd.optString("source");
//        ad.source = source;
//        if (!TextUtils.isEmpty(source) && !"gdt".equals(source)) {
//            ad.showAdTitle = true;
//        }
//
//        int adType = Utils.getSp().getInt(AD_TYPE, 2);
//        if (adType == ConstantPool.AdType.INSERT.getType()) {//插屏图文显示
//            ad.creative_type = 3;
//            Ad.Mix mix = new Ad.Mix();
//            ad.mix = mix;
//            mix.title = ad.title;
//            mix.sub_title = ad.des;
//            mix.imgurl = ad.imgurl;
//        }
//
//        //video
//        JSONObject videoObject = jsonAd.optJSONObject("video");
//        if (videoObject != null) {
//            ad.reward_video = new Ad.RewardVideoAds();
//            ad.reward_video.videoUrl = videoObject.optString("url");
//            ad.reward_video.icon = ad.lgsrc;
//            ad.reward_video.title = ad.title;
//            ad.reward_video.subTitle = ad.des;
//            int width = videoObject.optInt("w");
//            int height = videoObject.optInt("h");
//            ad.reward_video.videoWidth = String.valueOf(width);
//            ad.reward_video.videoHeight = String.valueOf(height);
//            ad.reward_video.orientation = SCREEN_ORIENTATION_PORTRAIT;
//            if (width > height) {//横屏
//                ad.reward_video.orientation = SCREEN_ORIENTATION_LANDSCAPE;
//            }
//            String durationStr = String.valueOf(videoObject.optInt("duration"));
//            if (TextUtils.isEmpty(durationStr) || "0".equals(durationStr)) {
//                durationStr = getPlayTime(ad.reward_video.videoUrl);
//            }
//            ad.reward_video.videoDuration = durationStr;
//        }
//        ad.trackerGroup.put(ConstantPool.TrackerType.DISPLAY, add2List(jsonAd, "imp_tracks"));//曝光追踪
//        ad.trackerGroup.put(ConstantPool.TrackerType.CLICK, add2List(jsonAd, "click_tracks"));//点击追踪
//        ad.trackerGroup.put(ConstantPool.TrackerType.CLOSE, add2List(jsonAd, "ad_close_tracks"));//点击追踪
//        ad.trackerGroup.put(ConstantPool.TrackerType.DEEP_LINK_OPEN, add2List(jsonAd, "dp_tracks"));//Deeplink唤起
//        ad.trackerGroup.put(ConstantPool.TrackerType.MUTE_TRACKER, add2List(jsonAd, "video_unmute_tracks"));//取消静音
//        ad.trackerGroup.put(ConstantPool.TrackerType.UNMUTE_TRACKER, add2List(jsonAd, "video_mute_tracks"));//静音.
//        ad.trackerGroup.put(ConstantPool.TrackerType.VIDEO_START_TRACKER, add2List(jsonAd, "video_start_tracks"));//播放开始.
//        ad.trackerGroup.put(ConstantPool.TrackerType.VIDEO_CLICK_TRACKER, add2List(jsonAd, "video_start_tracks"));//播放点击.
//        ad.trackerGroup.put(ConstantPool.TrackerType.VIDEO_PAUSE_TRACKER, add2List(jsonAd, "pauseTracker"));//播放暂停.
//        ad.trackerGroup.put(ConstantPool.TrackerType.VIDEO_COMPLETE_TRACKER, add2List(jsonAd, "video_complete_tracks"));//播放完成.
//        ad.trackerGroup.put(ConstantPool.TrackerType.VIDEO_SKIP_TRACKER, add2List(jsonAd, "video_skip_tracks"));//视频跳过播放.
//        ad.trackerGroup.put(ConstantPool.TrackerType.VIDEO_PLAYING_1_TRACKER, add2List(jsonAd, "video_play_first_quartile_tracks"));//视频播放1/4.
//        ad.trackerGroup.put(ConstantPool.TrackerType.VIDEO_PLAYING_2_TRACKER, add2List(jsonAd, "video_play_midpoint_tracks"));//视频播放1/2.
//        ad.trackerGroup.put(ConstantPool.TrackerType.VIDEO_PLAYING_3_TRACKER, add2List(jsonAd, "video_play_third_quartile_tracks"));//视频播放3/4.
//        ad.trackerGroup.put(ConstantPool.TrackerType.VIDEO_PLAYING_3_TRACKER, add2List(jsonAd, "video_play_third_quartile_tracks"));//视频播放3/4.

        return ad;
    }

    private String getPlayTime(String mUri) {
        MediaMetadataRetriever mmr = new MediaMetadataRetriever();
        try {
            if (mUri != null) {
                mmr.setDataSource(mUri);
            }
            String durationStr = mmr.extractMetadata(MediaMetadataRetriever.METADATA_KEY_DURATION);//时长(毫秒)
            if (!TextUtils.isEmpty(durationStr)) {
                return String.valueOf((int) Integer.parseInt(durationStr) / 1000);
            }
        } catch (Exception ex) {
            Log.e("TAG", "MediaMetadataRetriever exception " + ex);
        } finally {
            try {
                mmr.release();
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }
        return "0";
    }


    /**
     * 解析String泛型的list
     */
    private String[] add2List(JSONObject jsonObject, String arrayKey) {

        String[] dataArray;
        try {
            JSONArray jsonArray = jsonObject.optJSONArray(arrayKey);
            if (jsonArray == null || jsonArray.length() == 0) {
                return null;
            }
            int length = jsonArray.length();
            dataArray = new String[length];
            for (int i = 0; i < length; i++) {
                dataArray[i] = (String) jsonArray.opt(i);
            }
            return dataArray;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 解析String泛型的list
     */
    private String[] addToList(JSONObject jsonObject, String arrayKey, String arrayKey2) {

        String[] dataArray;
        try {
            JSONArray jsonArray = jsonObject.getJSONArray(arrayKey);
            JSONArray jsonArray2 = jsonObject.getJSONArray(arrayKey2);
            if (jsonArray2 == null) {
                int length = jsonArray.length();
                dataArray = new String[length];
                for (int i = 0; i < length; i++) {
                    dataArray[i] = (String) jsonArray.opt(i);
                }
            } else {
                int length = jsonArray.length();
                int length2 = jsonArray2.length();
                dataArray = new String[length + length2];
                for (int i = 0; i < length; i++) {
                    dataArray[i] = (String) jsonArray.opt(i);
                }
                for (int i = 0; i < length2; i++) {
                    dataArray[i + length] = (String) jsonArray2.opt(i);
                }
            }
            return dataArray;
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return null;
    }
}
