package com.halomobi.network.request.ad;

import android.media.MediaMetadataRetriever;
import android.text.TextUtils;
import android.util.Log;

import com.halomobi.common.ConstantPool;
import com.halomobi.network.listeners.AdRedirectChangListener;
import com.halomobi.network.listeners.AdStateChangListener;
import com.halomobi.network.net.AbsNetProcessor;
import com.halomobi.network.net.AbsNetTask;
import com.halomobi.network.net.http.HttpResult;
import com.halomobi.network.request.ad.bean.Ad;
import com.halomobi.network.request.ad.bean.AdParameter;
import com.halomobi.utils.LogUtils;
import com.halomobi.utils.NetworkUtils;
import com.halomobi.utils.Utils;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.IOException;
import java.util.ArrayList;

public class RequestRedirectAdProcessor extends AbsNetProcessor {
    private String mUrlStr;
    private AdRedirectChangListener mListener;

    RequestRedirectAdProcessor(String urlStr, AdRedirectChangListener listener) {
        mListener = listener;
        mUrlStr = urlStr;
    }

    @Override
    public void onNetworkError() {
        onAdLoadFailed("网络异常", ConstantPool.EroType.WITHOUT_NET);
    }

    @Override
    public void onErrorReceived(String eroMsg, int eroCode) {
        onAdLoadFailed(eroMsg, eroCode);
    }

    @Override
    public void onDataReceived(HttpResult result) {
        mListener.onXmlSuccess(result.getString());
    }

    @Override
    protected AbsNetTask createNetTask() {
        return new RequestRedirectAdTask(mUrlStr);
    }

    private void onAdLoadFailed(final String errorMsg, final int errorCode) {
        Utils.post(() -> {
            LogUtils.d("onAdLoadFailed errorCode:" + errorCode);
            onAdError(errorMsg, errorCode);
        });
    }

    @Override
    public void process() {
        if (NetworkUtils.getNetworkType() == NetworkUtils.NET_NONE) {
            onAdLoadFailed("无网络", ConstantPool.EroType.WITHOUT_NET);
            return;
        }

        if (TextUtils.isEmpty(mUrlStr)) {
            onAdLoadFailed("广告 id 为空", ConstantPool.EroType.PARAMS_ERO);
            return;
        }

        super.process();

    }

    /**
     * 广告请求失败
     *
     * @param errorMsg  错误信息
     * @param errorCode 错误码
     */
    public void onAdError(String errorMsg, int errorCode) {
        if (mListener != null) {
            mListener.onAdError(errorMsg, errorCode);
        }
    }

}
