package com.halomobi.utils;

import android.graphics.Bitmap;
import android.graphics.Bitmap.Config;
import android.graphics.Canvas;
import android.graphics.Color;
import android.graphics.Paint;
import android.graphics.PorterDuff.Mode;
import android.graphics.PorterDuffXfermode;
import android.graphics.Rect;
import android.graphics.RectF;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.GradientDrawable;
import android.graphics.drawable.StateListDrawable;
import android.view.View;
import android.view.animation.Animation;
import android.view.animation.AnimationUtils;

public class DrawableUtils {

    /**
     * 创建一个图片
     *
     * @param contentColor 内部填充颜色
     * @param strokeColor  描边颜色
     * @param radius       圆角
     */
    public static GradientDrawable createDrawable(int contentColor,
                                                  int strokeColor, int radius) {
        GradientDrawable drawable = new GradientDrawable(); // 生成Shape
        drawable.setGradientType(GradientDrawable.LINEAR_GRADIENT); // 设置矩形
        drawable.setColor(contentColor);// 内容区域的颜色
        drawable.setStroke(1, strokeColor); // 四周描边,描边后四角真正为圆角，不会出现黑色阴影。如果父窗体是可以滑动的，需要把父View设置setScrollCache(false)
        drawable.setCornerRadius(radius); // 设置四角都为圆角
        return drawable;
    }

    /**
     * 创建一个图片选择器
     *
     * @param normalState  普通状态的图片
     * @param pressedState 按压状态的图片
     */
    public static StateListDrawable createSelector(Drawable normalState,
                                                   Drawable pressedState) {
        StateListDrawable bg = new StateListDrawable();
        if (pressedState == null) {
            pressedState = normalState.getConstantState().newDrawable();
            pressedState.setColorFilter(Color.parseColor("#30000000"), Mode.MULTIPLY);
        }
        bg.addState(new int[]{android.R.attr.state_pressed,
                android.R.attr.state_enabled}, pressedState);
        bg.addState(new int[]{android.R.attr.state_enabled}, normalState);
        bg.addState(new int[]{}, normalState);
        return bg;
    }

    public static StateListDrawable createSelector(int normalColor,
                                                   int pressedColro) {
        Drawable normalState = createDrawable(normalColor, normalColor, 0);
        Drawable pressedState = createDrawable(pressedColro, pressedColro, 0);
        return createSelector(normalState, pressedState);
    }

//	/** 获取图片的大小 */
//	@SuppressLint("NewApi")
//	public static int getDrawableSize(Drawable drawable) {
//		if (drawable == null) {
//			return 0;
//		}
//		Bitmap bitmap = ((BitmapDrawable) drawable).getBitmap();
//		if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.HONEYCOMB_MR1) {
//			return bitmap.getByteCount();
//		} else {
//			return bitmap.getRowBytes() * bitmap.getHeight();
//		}
//	}

    /**
     * 获得“x”的path路径
     *
     * @param roundX 圆心的X坐标
     * @param roundY 圆心的Y坐标
     * @param roundR 圆的半径
     * @return
     */
    public static float[] getXPath(float roundX, float roundY, float roundR) {

        float[] point = {roundX - roundR / 2, roundY - roundR / 2,
                roundX + roundR / 2, roundY + roundR / 2, roundX - roundR / 2,
                roundY + roundR / 2, roundX + roundR / 2, roundY - roundR / 2};
        return point;
    }

    /**
     * 设置关闭区域
     */
    public static RectF getArea(View v, RectF r, int rang) {
        float left = Math.max(0, r.left - rang);
        float top = Math.max(0, r.top - rang);
        float right = Math.min(v.getWidth(), r.right + rang);
        float bottom = Math.min(v.getHeight(), r.bottom + rang);
        return new RectF(left, top, right, bottom);
    }


    public static Animation getDisplayAnim() {
        return AnimationUtils.loadAnimation(Utils.getContext(),
                android.R.anim.fade_in);
    }

    //生成圆角图片
    public static Bitmap getRoundedCornerBitmap(Bitmap bitmap, float roundPx) {
        try {
            Bitmap output = Bitmap.createBitmap(bitmap.getWidth(),
                    bitmap.getHeight(), Config.ARGB_8888);
            Canvas canvas = new Canvas(output);
            final Paint paint = new Paint();
            final Rect rect = new Rect(0, 0, bitmap.getWidth(), bitmap.getHeight());
            final RectF rectF = new RectF(new Rect(0, 0, bitmap.getWidth(), bitmap.getHeight()));
            paint.setAntiAlias(true);
            canvas.drawARGB(0, 0, 0, 0);
            paint.setColor(Color.BLACK);
            canvas.drawRoundRect(rectF, roundPx, roundPx, paint);
            paint.setXfermode(new PorterDuffXfermode(Mode.SRC_IN));
            final Rect src = new Rect(0, 0, bitmap.getWidth(), bitmap.getHeight());
            canvas.drawBitmap(bitmap, src, rect, paint);
            return output;
        } catch (Exception e) {
            return bitmap;
        }
    }

}
