package com.halomobi.utils;

import static android.content.Context.WIFI_SERVICE;
import android.content.Context;
import android.net.ConnectivityManager;
import android.net.NetworkInfo;
import android.net.wifi.WifiInfo;
import android.net.wifi.WifiManager;
import android.telephony.TelephonyManager;
import android.text.TextUtils;

import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.net.NetworkInterface;
import java.util.Collections;
import java.util.List;

public class NetworkUtils {

    public static final int NET_NONE = -1;
    public static final int NET_UNKNOWN = 0x00;
    public static final int NET_WIFI = 0x02;
    public static final int NET_CELL_UNKNOWN = 0x03;
    public static final int NET_2G = 0x04;
    public static final int NET_3G = 0x05;
    public static final int NET_4G = 0x06;

    /**
     * 判断当前是否有网
     * <p>
     * 1 Wifi
     * <p>
     * 2 4G
     * <p>
     * 3 3G
     * <p>
     * 4 2G
     * <p>
     * 0 无网
     *
     * @return 判断值
     */
    public static int getNetType() {

        ConnectivityManager cwjManager = (ConnectivityManager) Utils
                .getContext().getSystemService(Context.CONNECTIVITY_SERVICE);
        if (cwjManager != null) {
            NetworkInfo info = cwjManager.getActiveNetworkInfo();
            if (info == null) {
                return 0;// 无网
            }
            int netType = info.getType();
            if (netType == ConnectivityManager.TYPE_WIFI) {
                return 2;// wifi
            } else if (netType == ConnectivityManager.TYPE_MOBILE) {
                switch (info.getSubtype()) {
                    case TelephonyManager.NETWORK_TYPE_GPRS://2G
                    case TelephonyManager.NETWORK_TYPE_EDGE:
                    case TelephonyManager.NETWORK_TYPE_CDMA:
                    case TelephonyManager.NETWORK_TYPE_1xRTT:
                    case TelephonyManager.NETWORK_TYPE_IDEN:
                        return 3;
                    case TelephonyManager.NETWORK_TYPE_UMTS:
                    case TelephonyManager.NETWORK_TYPE_EVDO_0:
                    case TelephonyManager.NETWORK_TYPE_EVDO_A:
                    case TelephonyManager.NETWORK_TYPE_HSDPA:
                    case TelephonyManager.NETWORK_TYPE_HSUPA:
                    case TelephonyManager.NETWORK_TYPE_HSPA:
                    case TelephonyManager.NETWORK_TYPE_EVDO_B:
                    case TelephonyManager.NETWORK_TYPE_EHRPD:
                    case TelephonyManager.NETWORK_TYPE_HSPAP:
                        return 4;
                    case TelephonyManager.NETWORK_TYPE_LTE://4G
                        return 5;
                    default:
                        return 0;
                }
            }
        }
        return 0;
    }

    /**
     * 当前网络是否可用
     *
     * @param cxt Context
     * @return activeNetInfo.isConnected();
     */
    public static boolean isNetworkConnected(Context cxt) {
        ConnectivityManager connectivityManager = (ConnectivityManager) cxt
                .getApplicationContext().getSystemService(
                        Context.CONNECTIVITY_SERVICE);

        NetworkInfo activeNetInfo = connectivityManager.getActiveNetworkInfo();
        if (activeNetInfo != null) {
            return activeNetInfo.isConnected();
        }
        return false;
    }

    /**
     * 判断当前是否有网
     * <p>
     * 2 Wifi
     * <p>
     * 6 4G
     * <p>
     * 5 3G
     * <p>
     * 4 2G
     * <p>
     * -1 无网
     *
     * @return 判断值
     */
    public static int getNetworkType() {

        ConnectivityManager cwjManager = (ConnectivityManager) Utils
                .getContext().getSystemService(Context.CONNECTIVITY_SERVICE);
        if (cwjManager != null) {
            NetworkInfo info = cwjManager.getActiveNetworkInfo();
            if (info == null) {
                return NET_NONE;// 无网
            }
            int netType = info.getType();
            if (netType == ConnectivityManager.TYPE_WIFI) {
                return NET_WIFI;// wifi
            } else if (netType == ConnectivityManager.TYPE_MOBILE) {
                return getMobileNetClass(info.getSubtype());// 各种手机网络
            }
        }
        return 0;
    }

    /**
     * 区分手机制式
     *
     * @param networkType
     * @return
     */
    private static int getMobileNetClass(int networkType) {

        switch (networkType) {
            case TelephonyManager.NETWORK_TYPE_GPRS:
            case TelephonyManager.NETWORK_TYPE_EDGE:
            case TelephonyManager.NETWORK_TYPE_CDMA:
            case TelephonyManager.NETWORK_TYPE_1xRTT:
            case TelephonyManager.NETWORK_TYPE_IDEN:
                return NET_2G;
            case TelephonyManager.NETWORK_TYPE_UMTS:
            case TelephonyManager.NETWORK_TYPE_EVDO_0:
            case TelephonyManager.NETWORK_TYPE_EVDO_A:
            case TelephonyManager.NETWORK_TYPE_HSDPA:
            case TelephonyManager.NETWORK_TYPE_HSUPA:
            case TelephonyManager.NETWORK_TYPE_HSPA:
            case TelephonyManager.NETWORK_TYPE_EVDO_B:
            case TelephonyManager.NETWORK_TYPE_EHRPD:
            case TelephonyManager.NETWORK_TYPE_HSPAP:
                return NET_3G;
            case TelephonyManager.NETWORK_TYPE_LTE:
                return NET_4G;
            default:
                return NET_UNKNOWN;
        }
    }


    /**
     * 获取当前设备的MAC地址
     */
    public static String getMacAddress() {
        String mac = SystemUtils.getInfoByJson("mac");
        return mac;
//        if (!TextUtils.isEmpty(mac)) return mac;
//        if (SystemUtils.getSDK_INT() >= 23) {
//            return getWifiMacAddress();
//        } else {
//            return getMacInNormal();
//        }
    }

    /**
     * 获取wifi mac地址
     *
     * @return wifi mac地址
     */
    public static String getWifiMacAddress() {
        try {
            String interfaceName = "wlan0";
            List<NetworkInterface> interfaces = Collections.list(NetworkInterface.getNetworkInterfaces());
            for (NetworkInterface intf : interfaces) {
                if (!intf.getName().equalsIgnoreCase(interfaceName)) {
                    continue;
                }
                byte[] mac = new byte[0];
                if (android.os.Build.VERSION.SDK_INT >= android.os.Build.VERSION_CODES.GINGERBREAD) {
                    mac = intf.getHardwareAddress();
                }
                if (mac == null) {
                    return "";
                }
                StringBuilder buf = new StringBuilder();
                for (byte aMac : mac) {
                    buf.append(String.format("%02X:", aMac));
                }
                if (buf.length() > 0) {
                    buf.deleteCharAt(buf.length() - 1);
                }
                return buf.toString();
            }
        } catch (Exception ex) {
        } // for now eat exceptions
        return "";
    }

    private static String getMacInNormal() {
        String mac = "";
        Context context = Utils.getContext();
        if (null != context) {
            try {
                WifiManager wm = (WifiManager) context.getSystemService(WIFI_SERVICE);
                WifiInfo info = wm.getConnectionInfo();
                mac = info.getMacAddress();
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return mac;
    }


    private static String getMacInAndroidM() {
        ProcessBuilder cmd = null;
        Process process = null;
        BufferedReader reader = null;
        try {
            String[] args = {"/system/bin/cat", "/sys/class/net/wlan0/address"};
            cmd = new ProcessBuilder(args);
            process = cmd.start();
            reader = new BufferedReader(new InputStreamReader(process.getInputStream(), "UTF-8"));
            String line = reader.readLine();
            return line;
        } catch (Exception ex) {
            ex.printStackTrace();
        } finally {
            FileUtils.close(reader);
            if (process != null) {
                process.destroy();
            }
        }
        return "";
    }

    /**
     * 获取Wifi ID
     *
     * @return
     */
    public static String getNetWorkID() {
        try {
            WifiManager wifi_service = (WifiManager) Utils.getContext().getApplicationContext().getSystemService(WIFI_SERVICE);
            WifiInfo wifiInfo = wifi_service.getConnectionInfo();
            return wifiInfo.getNetworkId() + "";
        } catch (Exception e) {
            return "nul";
        }
    }

    /**
     * 获取局域网名称
     *
     * @return
     */
    public static String getWifiMac() {
        try {
            WifiManager wifi_service = (WifiManager) Utils.getContext().getApplicationContext().getSystemService(WIFI_SERVICE);
            WifiInfo wifiInfo = wifi_service.getConnectionInfo();
            return wifiInfo.getMacAddress();
        } catch (Exception e) {
            return "nul";
        }
    }

    /**
     * 获取局域网名称
     *
     * @return
     */
    public static String getSSID() {
        try {
            WifiManager wifi_service = (WifiManager) Utils.getContext().getApplicationContext().getSystemService(WIFI_SERVICE);
            WifiInfo wifiInfo = wifi_service.getConnectionInfo();
            return wifiInfo.getSSID();
        } catch (Exception e) {
            return "nul";
        }
    }

    /**
     * 连接速度
     *
     * @return
     */
    public static int getLksd() {
        try {
            WifiManager wifi_service = (WifiManager) Utils.getContext().getApplicationContext().getSystemService(WIFI_SERVICE);
            WifiInfo wifiInfo = wifi_service.getConnectionInfo();
            return wifiInfo.getLinkSpeed();
        } catch (Exception e) {
            return 0;
        }
    }

    /**
     * 接收信号强度
     *
     * @return
     */
    public static int getRssi() {
        try {
            WifiManager wifi_service = (WifiManager) Utils.getContext().getApplicationContext().getSystemService(WIFI_SERVICE);
            WifiInfo wifiInfo = wifi_service.getConnectionInfo();
            return wifiInfo.getRssi();
        } catch (Exception e) {
            return 0;
        }
    }

    /**
     * 是否漫游
     *
     * @return
     */
    public static int isRoaming() {
        try {
            ConnectivityManager manager = (ConnectivityManager) Utils.getContext().getSystemService(Context.CONNECTIVITY_SERVICE);
            NetworkInfo info = manager.getActiveNetworkInfo();
            if (info == null || !info.isConnected()) {
                return 0;
            }
            if (info.isRoaming()) {
                // here is the roaming option you can change it if you want to
                // disable internet while roaming, just return false
                return 1;
            } else {
                return 0;
            }
        } catch (Exception e) {
            e.printStackTrace();
            return 0;
        }
    }

}
