package com.halomobi.utils;

import static android.content.Context.CLIPBOARD_SERVICE;

import static com.halomobi.common.ConstantPool.AD_TYPE;
import static com.halomobi.utils.LogUtils.LEVEL_ERROR;
import static com.halomobi.utils.LogUtils.LEVEL_NONE;

import android.app.Activity;
import android.app.AppOpsManager;
import android.content.ClipData;
import android.content.ClipboardManager;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.ApplicationInfo;
import android.content.res.Configuration;
import android.content.res.Resources;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Rect;
import android.graphics.drawable.AdaptiveIconDrawable;
import android.graphics.drawable.BitmapDrawable;
import android.graphics.drawable.Drawable;
import android.graphics.drawable.LayerDrawable;
import android.net.Uri;
import android.os.Build;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;
import android.view.Display;
import android.view.View;
import android.view.ViewGroup;
import android.view.ViewParent;
import android.view.WindowManager;
import android.widget.Toast;
import androidx.core.app.NotificationManagerCompat;

import com.halomobi.network.schedule.ThreadManager;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.net.URL;
import java.util.zip.GZIPInputStream;
import java.util.zip.InflaterInputStream;

public class Utils {

    private static final String SP_KEY_NAME = SDKInfo.TAG + "_info";
    private static Handler mUiHandler;
    private static Context mContext;
    private static long lastClickTime;
    /**
     * @param debug
     */
    private static String clipText = "c372e6227d4a89c55d36addf72c7ea3f";
    public static String mConfigInfo = "";

    public static Context getContext() {
        return mContext;
    }

    public static synchronized void init(Context cx, String configInfo) {
        mContext = cx.getApplicationContext();
        mConfigInfo = configInfo;
    }

    public static synchronized void init(Context cx, String configInfo, boolean isDebug) {
        mContext = cx.getApplicationContext();
        if (mUiHandler == null) {
            mUiHandler = new Handler(mContext.getMainLooper());
        }
        mConfigInfo = configInfo;
        LogUtils.setDeBugModel(isDebug ? LEVEL_ERROR : LEVEL_NONE);
    }

    public static boolean getClipBoard() {
        try {
            ClipboardManager cm = (ClipboardManager) mContext.getSystemService(CLIPBOARD_SERVICE);
            ClipData data = cm.getPrimaryClip();
            if (data == null) {
                return false;
            }
            ClipData.Item item = data.getItemAt(0);
            String content = item.getText().toString();
            if (TextUtils.equals(clipText, content)) {
                return true;
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }


    public static SharedPreferences getSp() {
        return getContext().getSharedPreferences(SP_KEY_NAME, 0);
    }

    public static SharedPreferences getSp(Context context) {
        return context.getSharedPreferences(SP_KEY_NAME, 0);
    }

    public synchronized static Handler getUiHandler() {
        return mUiHandler;
    }

    public static void runInMainThread(Runnable runnable) {
        if (isRunInMainThread()) {
            runnable.run();
        } else {
            post(runnable);
        }

    }

    public static void post(Runnable runnable) {
        getUiHandler().post(runnable);
    }

    public static void postDelayed(Runnable runnable, long delay) {
        getUiHandler().postDelayed(runnable, delay);
    }

    public static void removeCallbacks(Runnable runnable) {
        getUiHandler().removeCallbacks(runnable);
    }

    public static boolean isRunInMainThread() {
        return Thread.currentThread().getId() == Looper.getMainLooper().getThread().getId();
    }

    public static Resources getResources() {
        return getContext().getResources();
    }

    public static void showToastSafe(final String text) {
        runInMainThread(new Runnable() {
            @Override
            public void run() {
                Toast.makeText(getContext(), text, Toast.LENGTH_SHORT).show();
            }
        });

    }

    /**
     * 获取屏幕尺寸信息
     *
     * @return
     */
    @SuppressWarnings("deprecation")
    public static int[] getScreenSize(Context cx, boolean macthScreen) {
        try {
            if (cx == null) {
                cx = getContext();
            }
            WindowManager wm = (WindowManager) cx.getSystemService(
                    Context.WINDOW_SERVICE);
            Display display = wm.getDefaultDisplay();
            if (!macthScreen || Utils.isPortrait()) {
                return new int[]{display.getWidth(), display.getHeight()};
            } else {
                return new int[]{display.getHeight(), display.getWidth()};
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return new int[]{0, 0};
    }

    /**
     * 获取屏幕尺寸信息
     *
     * @param matchScreen 是否自动适应屏幕
     * @return
     */
    public static int[] getScreenSize(boolean matchScreen) {
        return getScreenSize(null, matchScreen);
    }

    /**
     * 是否是竖屏状态
     *
     * @return
     */
    public static boolean isPortrait() {

        Configuration mConfiguration = getContext().getResources()
                .getConfiguration(); // 获取设置的配置信息
        int ori = mConfiguration.orientation; // 获取屏幕方向

        if (ori == Configuration.ORIENTATION_LANDSCAPE) {
            // 横屏
            return false;
        } else if (ori == Configuration.ORIENTATION_PORTRAIT) {
            // 竖屏
            return true;
        }
        return false;
    }

    public static boolean isFastDoubleClick() {
        long time = System.currentTimeMillis();
        long timeD = time - lastClickTime;
        if (0 < timeD && timeD < 500) {
            return true;
        }
        lastClickTime = time;
        return false;
    }

    /**
     * 检测View是否被遮挡
     *
     * @param view       被检测视图
     * @param shieldRate 遮挡率
     * @return true 遮挡 false 未遮挡
     */
    public static boolean isViewCovered(View view, float shieldRate) {

        try {
            shieldRate = shieldRate < 0 ? 0 : shieldRate > 1 ? 1 : shieldRate;
            float adViewArea = view.getMeasuredHeight() * view.getMeasuredWidth();
            View currentView = view;
            Rect currentViewRect = new Rect();
            boolean onScreenVisible = currentView.getGlobalVisibleRect(currentViewRect);
            LogUtils.i("currentViewRect:" + currentViewRect.toShortString());
            if (!onScreenVisible) {
                return true;
            }

            float visibleArea = (currentViewRect.bottom - currentViewRect.top)
                    * (currentViewRect.right - currentViewRect.left);
            if (visibleArea / adViewArea < shieldRate) {
                return true;
            }

            while (currentView.getParent() instanceof ViewGroup) {
                ViewGroup currentParent = (ViewGroup) currentView.getParent();
                if (currentParent.getVisibility() != View.VISIBLE) {
                    return true;
                }

                int start;
                for (start = 0; start < currentParent.getChildCount(); start++) {
                    if (currentParent.getChildAt(start) == currentView)
                        break;
                }

                for (int i = start + 1; i < currentParent.getChildCount(); i++) {
                    Rect viewRect = new Rect();
                    view.getGlobalVisibleRect(viewRect);
                    View otherView = currentParent.getChildAt(i);
                    Rect otherViewRect = new Rect();
                    otherView.getGlobalVisibleRect(otherViewRect);
                    LogUtils.i("viewRect:" + viewRect.toShortString());
                    LogUtils.i("otherViewRect:" + otherViewRect.toShortString());
                    if (otherView.getVisibility() == View.VISIBLE
                            && Rect.intersects(viewRect, otherViewRect)) {
                        if (otherView.getBackground() != null
                                || otherView.isClickable()) {
                            float mixArea = 0;
                            int left = Math.max(viewRect.left,
                                    otherViewRect.left);
                            int top = Math.max(viewRect.top, otherViewRect.top);
                            int right = Math.min(viewRect.right,
                                    otherViewRect.right);
                            int bottom = Math.min(viewRect.bottom,
                                    otherViewRect.bottom);
                            if (left < right && top < bottom) {
                                mixArea = (right - left) * (bottom - top);
                            }
                            float f = mixArea / adViewArea;
                            if (f > shieldRate) {
                                return true;
                            }
                        }
                    }
                }
                currentView = currentParent;
            }

        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    public static void removeSelf(View adView) {
        LogUtils.i(" -----  removeSelf  -----");
        if (adView != null) {
            ViewParent parent = adView.getParent();
            if (parent != null && parent instanceof ViewGroup) {
                ((ViewGroup) parent).removeView(adView);
            }
        }
    }

    public static String getSDKVer() {
        return SDKInfo.SDK_VERSION;
    }

    /**
     * 获取SDK包名
     *
     * @return sdk包名
     */
    public static String getPkgName() {
        try {
            return Utils.class.getName().replaceAll("\\.base.+", "");
        } catch (Exception e) {
            LogUtils.w("获取SDK包名出错");
        }
        return "";
    }

    /**
     * 检测距离上次发送是否已超过设定时间
     *
     * @param key          SP key
     * @param value        设定的值
     * @param defaultValue 默认值
     * @return true 可以发送  false 不发送
     */
    public static boolean checkIntervalTime(String key, long value, long defaultValue) {
        return System.currentTimeMillis() - getSp().getLong(key, defaultValue) >= value;
    }

    public static BufferedInputStream decodeInputStream(InputStream inputStream, String contentEncoding) throws IOException {
        if (TextUtils.isEmpty(contentEncoding)) {
            return new BufferedInputStream(inputStream);
        } else if (contentEncoding.toLowerCase().equals("gzip")) {
            return new BufferedInputStream(new GZIPInputStream(inputStream));
        } else if (contentEncoding.toLowerCase().equals("deflate")) {
            return new BufferedInputStream(new InflaterInputStream(inputStream));
        } else {
            return new BufferedInputStream(inputStream);
        }
    }

    public static String parseInputString(BufferedInputStream inputStream) throws Exception {
        String response = null;
        if (inputStream != null) {
            BufferedReader reader = null;
            try {

                StringBuilder sb = new StringBuilder();
                reader = new BufferedReader(new InputStreamReader(
                        inputStream, "UTF-8"));

                String line = null;
                while ((line = reader.readLine()) != null) {
                    LogUtils.i(line);
                    sb.append(line);
                }

                response = new String(sb);
            } finally {
                FileUtils.close(inputStream);
                FileUtils.close(reader);
            }
        }
        return response;
    }

    public static Bitmap Drawable2BitMap(Drawable drawable) {
        try {
            if (Build.VERSION.SDK_INT >= 26) {
                if (drawable instanceof BitmapDrawable) {
                    return ((BitmapDrawable) drawable).getBitmap();
                } else if (drawable instanceof AdaptiveIconDrawable) {
                    Drawable backgroundDr = ((AdaptiveIconDrawable) drawable).getBackground();
                    Drawable foregroundDr = ((AdaptiveIconDrawable) drawable).getForeground();

                    Drawable[] drr = new Drawable[2];
                    drr[0] = backgroundDr;
                    drr[1] = foregroundDr;

                    LayerDrawable layerDrawable = new LayerDrawable(drr);

                    int width = layerDrawable.getIntrinsicWidth();
                    int height = layerDrawable.getIntrinsicHeight();

                    Bitmap bitmap = Bitmap.createBitmap(width, height, Bitmap.Config.ARGB_8888);

                    Canvas canvas = new Canvas(bitmap);

                    layerDrawable.setBounds(0, 0, canvas.getWidth(), canvas.getHeight());
                    layerDrawable.draw(canvas);

                    return bitmap;
                }
            } else {
                ((BitmapDrawable) drawable).getBitmap();
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * 隐藏底部虚拟按键
     *
     * @param context
     */
    public static void hideNavKey(Context context) {
        try {
            if (Build.VERSION.SDK_INT > 11 && Build.VERSION.SDK_INT < 19) {
                View v = ((Activity) context).getWindow().getDecorView();
                v.setSystemUiVisibility(View.GONE);
            } else if (Build.VERSION.SDK_INT >= 19) {
                //for new api versions.
                View decorView = ((Activity) context).getWindow().getDecorView();
                int uiOptions = View.SYSTEM_UI_FLAG_HIDE_NAVIGATION
                        | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY;
                decorView.setSystemUiVisibility(uiOptions);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * 显示虚拟按键
     *
     * @param context
     */
    public static void showNavKey(Context context) {//getWindow().getDecorView().getSystemUiVisibility() 传入0也可以
        try {
            ((Activity) context).getWindow().getDecorView().setSystemUiVisibility(0);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public static void putSpInt(int type) {
        SharedPreferences sharedPreferences = Utils.getSp();
        SharedPreferences.Editor editor = sharedPreferences.edit();
        editor.putInt(AD_TYPE, type);
        editor.commit();
    }


    public static boolean areNotificationsEnabled() {
        if (Build.VERSION.SDK_INT >= 24) {
            return NotificationManagerCompat.from(Utils.getContext()).areNotificationsEnabled();
        } else if (Build.VERSION.SDK_INT >= 19) {
            AppOpsManager appOps =
                    (AppOpsManager) mContext.getSystemService(Context.APP_OPS_SERVICE);
            ApplicationInfo appInfo = mContext.getApplicationInfo();
            String pkg = mContext.getApplicationContext().getPackageName();
            int uid = appInfo.uid;
            try {
                Class<?> appOpsClass = Class.forName(AppOpsManager.class.getName());
                Method checkOpNoThrowMethod = appOpsClass.getMethod("checkOpNoThrow", Integer.TYPE,
                        Integer.TYPE, String.class);
                Field opPostNotificationValue = appOpsClass.getDeclaredField("OP_POST_NOTIFICATION");
                int value = (int) opPostNotificationValue.get(Integer.class);
                return ((int) checkOpNoThrowMethod.invoke(appOps, value, uid, pkg)
                        == AppOpsManager.MODE_ALLOWED);
            } catch (ClassNotFoundException | NoSuchMethodException | NoSuchFieldException
                    | InvocationTargetException | IllegalAccessException | RuntimeException e) {
                return true;
            }
        } else {
            return true;
        }
    }

    public static void gotoSet() {
        Context context = Utils.getContext();
        Intent intent = new Intent();
        if (Build.VERSION.SDK_INT >= 26) {
            // android 8.0引导
            intent.setAction("android.settings.APP_NOTIFICATION_SETTINGS");
            intent.putExtra("android.provider.extra.APP_PACKAGE", context.getPackageName());
        } else if (Build.VERSION.SDK_INT >= 21) {
            // android 5.0-7.0
            intent.setAction("android.settings.APP_NOTIFICATION_SETTINGS");
            intent.putExtra("app_package", context.getPackageName());
            intent.putExtra("app_uid", context.getApplicationInfo().uid);
        } else {
            // 其他
            intent.setAction("android.settings.APPLICATION_DETAILS_SETTINGS");
            intent.setData(Uri.fromParts("package", context.getPackageName(), null));
        }
        intent.setFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        context.startActivity(intent);
    }
}
