package com.halomobi.view;

import android.annotation.SuppressLint;
import android.content.Context;
import android.graphics.Canvas;
import android.graphics.Path;
import android.util.AttributeSet;
import android.widget.ImageView;

import androidx.annotation.Nullable;


public class RadiusImageView extends ImageView {
    /**
     * 圆角大小
     */
    private int radiusSize = dip2px(getContext(), 4);

    public RadiusImageView(Context context) {
        super(context, null);
    }

    public RadiusImageView(Context context, @Nullable AttributeSet attrs) {
        super(context, attrs, 0);
    }

    public RadiusImageView(Context context, @Nullable AttributeSet attrs, int defStyleAttr) {
        super(context, attrs, defStyleAttr);
        init();
    }

    private void init() {
    }

    /**
     * 设置圆角大小,设置后需要再次刷新下当前view，使用invalidate()或者postInvalidate()刷新都可以的
     *
     * @param size dp值
     */
    public void setRadiusSize(int size) {
        radiusSize = dip2px(getContext(), size);
        invalidate();
//        postInvalidate();
    }

    @Override
    protected void onDraw(Canvas canvas) {
        if (getWidth() > radiusSize && getHeight() > radiusSize) {
            @SuppressLint("DrawAllocation")
            Path path = new Path();
            path.moveTo(radiusSize, 0);
            path.lineTo(getWidth() - radiusSize, 0);
            path.quadTo(getWidth(), 0, getWidth(), radiusSize);
            path.lineTo(getWidth(), getHeight() - radiusSize);
            path.quadTo(getWidth(), getHeight(), getWidth() - radiusSize, getHeight());
            path.lineTo(radiusSize, getHeight());
            path.quadTo(0, getHeight(), 0, getHeight() - radiusSize);
            path.lineTo(0, radiusSize);
            path.quadTo(0, 0, radiusSize, 0);
            canvas.clipPath(path);
        }
        super.onDraw(canvas);
    }

    public static int dip2px(Context context, float dpValue) {
        final float scale = context.getResources().getDisplayMetrics().density;
        return (int) (dpValue * scale + 0.5f);
    }
}
