package com.halomobi.view.imageloader.cache;

import android.text.TextUtils;

import com.halomobi.utils.FileUtils;
import com.halomobi.utils.KeyUtil;
import com.halomobi.utils.LogUtils;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.OutputStream;

class DiskCache implements ImageCache {

    private static final long DISK_CACHE_SIZE = 1024 * 1024 * 50;

    private static final long CACHE_EXCEED_TIME = 60 * 60 * 24 * 7 * 1000;

    private DiskLruCache mDiskLruCache;

    DiskCache() {
        try {
            String iconDir = FileUtils.getIconDir();
            if (!TextUtils.isEmpty(iconDir)) {
                File iconDirs = new File(iconDir);
                deleteCache(iconDirs);
                mDiskLruCache = DiskLruCache.open(new File(iconDir), 1, 1, DISK_CACHE_SIZE);
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void deleteCache(File root) {
        if (root == null) {
            LogUtils.d("传入文件为空");
            return;
        }
        if (root.isDirectory()) {
            File[] files = root.listFiles();

            if (files == null) {
                LogUtils.w("files == null");
                return;
            }

            for (File file : files) {
                deleteCache(file.getAbsoluteFile());
            }
        } else {
            if (System.currentTimeMillis() - root.lastModified() > CACHE_EXCEED_TIME) {
                if (root.delete()) {
                    LogUtils.d("删除图片缓存成功");
                } else {
                    LogUtils.e("删除图片缓存失败");
                }
            }
        }
    }

    @Override
    public byte[] getCache(String url) {

        if (mDiskLruCache == null) {
            return null;
        }

        FileInputStream fis = null;
        try {
            DiskLruCache.Snapshot snapshot = mDiskLruCache.get(getCacheKey(url));
            if (snapshot != null) {
                fis = (FileInputStream) snapshot.getInputStream(0);
                return FileUtils.getByteFromStream(fis);
            }
        } catch (Exception e) {
            e.printStackTrace();
        } finally {
            FileUtils.close(fis);
        }
        return null;
    }

    @Override
    public void putCache(String url, byte[] value) {

        if (mDiskLruCache == null) {
            return;
        }

        DiskLruCache.Editor edit = null;
        OutputStream os = null;
        try {
            edit = mDiskLruCache.edit(getCacheKey(url));
            if (edit != null) {
                os = edit.newOutputStream(0);
                os.write(value, 0, value.length);
                edit.commit();
                mDiskLruCache.flush();
            }
        } catch (Exception e) {
            if (edit != null) {
                try {
                    edit.abort();
                } catch (IOException e1) {
                    e1.printStackTrace();
                }
            }
            e.printStackTrace();
        } finally {
            FileUtils.close(os);
        }
    }

    private String getCacheKey(String url) {
        return KeyUtil.getMD5(url);
    }

    @Override
    public void clear() {
        // nothing
    }
}
